// NEOPIXEL BLINKS WHEN RECEIVES HEARTBEAT
// TeZ - 2017
// Connect NEOPIXEL VCC > 5V, GND > G, IN > D4
// Connect Motor  to Wemos 3V3, G and D6

#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <OSCMessage.h>
#include <Metro.h>
#include <NeoPixelBus.h>
#include <String.h>
#include "wifisettings.h"

int SENDER = 4;  // here write the number of the sender you want to respond to

int device_id = -1;
int WIP[]={0,0,0,0};
int hold = 1;
bool firstime = true;
bool neopixstate = false;
unsigned long now = 0;
unsigned long lastbeat = 0;
int MOTORPIN = 12; // D6 on WEMOS


Metro alive = Metro(4230);



// neopixel params
const uint16_t PixelCount = 1; // how many neopixels in your strip?
const uint8_t PixelPin = 2;  // ignored for Esp8266 - Wemos, it uses GPIO2 =  PIN D4  (UART MODE)
#define colorSaturation 128
NeoPixelBus<NeoGrbFeature, NeoEsp8266Uart800KbpsMethod> strip(PixelCount, PixelPin);
RgbColor idc(10, 0, 0);
RgbColor red(colorSaturation, 0, 0);
RgbColor green(0, colorSaturation, 0);
RgbColor blue(0, 0, colorSaturation);
RgbColor white(colorSaturation);
RgbColor black(0);
HslColor hslRed(red);
HslColor hslGreen(green);
HslColor hslBlue(blue);
HslColor hslWhite(white);
HslColor hslBlack(black);
//


void setup()
{
    Serial.begin(115200);
    while (!Serial); // wait for serial attach

    // init network
    Serial.println();
    Serial.print("Connecting to ");
    Serial.println(ssid);
    
    WiFi.begin(ssid, pass);
    
    while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
    }
    
    Serial.println("");
    Serial.println("WiFi connected");  
    Serial.println("IP address: ");
    thisip = WiFi.localIP();
    Serial.println( thisip );
    
    Udp.begin(rxport);
    
    Serial.println("Starting UDP");
    Serial.print("Local port: ");
    Serial.println(Udp.localPort());
    
    device_id = WID; // thisip[3];
    //
    

    // init neopixel
    Serial.println();
    Serial.println("Initializing...");
    Serial.flush();
    // this resets all the neopixels to an off state
    strip.Begin();
    strip.SetPixelColor(0, idc);
    strip.Show();
    Serial.println();
    Serial.println(" Neopixel is running...");
    //

    pinMode(MOTORPIN, OUTPUT);
      
}



/////////////////////////////////////
void loop() {
  state_loop();
  osc_message_pump();
}

/////////////////////////////////////
void state_loop() {
  // send alive ACK message to show-control
  if(alive.check()) {
    OSCMessage out("/sos/ack");
    out.add( device_id );
    Udp.beginPacket(dest, txport);
    out.send(Udp);
    Udp.endPacket();
    out.empty();
  }

   if(firstime){sendOSCip(); firstime=false;}

  now = millis();
  if(abs (now - lastbeat) > 5000){    
     Serial.println("ELAPSED" + abs(now-lastbeat)); 
        strip.SetPixelColor(0, idc);
        strip.Show();
        lastbeat =now;
       
  }
  
  

}

/////////////////////////////////////
/// READ OSC MESSAGES ///
void osc_message_pump() {  
  OSCMessage in;
  int size;

  if( (size = Udp.parsePacket()) > 0)
  {
    Serial.println("processing OSC package");
    // parse incoming OSC message
    while(size--) {
      in.fill( Udp.read() );
    }

    if(!in.hasError()) {
     // in.route("/sos/ack", on_ack);
      in.route("/sos/sensor1", on_sensor1);
      in.route("/sos/beat", on_beat);
    }
  } // if
}


/////////////////////////////////////
void on_stop(OSCMessage &msg, int addrOffset) {
  Serial.println(">> CMD: STOP");
}

/////////////////////////////////////
void on_sensor1(OSCMessage &msg, int addrOffset) {
  Serial.print(">> CMD: SENSOR1 = " );
   int value;
  
  if( msg.isInt(0)) {
    value = msg.getInt(0);
  }

  Serial.println(value);
  
  digitalWrite(BUILTIN_LED, HIGH);   // turn the LED on (HIGH is the voltage level)
  delay(500);              // wait 
  digitalWrite(BUILTIN_LED, LOW);    // turn the LED off by making the voltage LOW
 
  

}

/////////////////////////////////////
void on_beat(OSCMessage &msg, int addrOffset) {

   
   int sender_id;
   float value; 
  if(msg.isInt(0)) {
    sender_id = msg.getInt(0);
    value = msg.getInt(1);
  }
  
 
  
  Serial.print(">> SENDER_id = " );
  Serial.print(sender_id);
  Serial.print(">> IBI = " );
  Serial.println(value);

if (sender_id == SENDER){
         lastbeat =now;
        
        if (neopixstate){
                           strip.SetPixelColor(0, green);
                           neopixstate = !neopixstate;
                           digitalWrite(MOTORPIN,HIGH);
                   }else{
                           strip.SetPixelColor(0, blue);
                            neopixstate = !neopixstate;
                            digitalWrite(MOTORPIN,LOW);
                        }
      
                            strip.Show();
    } else {        

                         digitalWrite(MOTORPIN,LOW);
    }


}

/////////////////////////////////////
void on_ack(OSCMessage &msg, int addrOffset) {
  Serial.println(">> OSC ACK");
}

/////////////////////////////////////
void sendOSCip(){

  WIP[0]=thisip[0];
  WIP[1]=thisip[1];
  WIP[2]=thisip[2];
  WIP[3]=thisip[3];


  OSCMessage out("/wemos/ip");
   out.add(WIP[0]);
   out.add(WIP[1]);
   out.add(WIP[2]);
   out.add(WIP[3]);
   Udp.beginPacket(dest, txport);
   out.send(Udp);
   Udp.endPacket();
   out.empty();

}



